/*

                       Postage Stamp Rasteriser
                  Special Edition for the Eat Watch

                            by John Walker
                       http://www.fourmilab.ch/

*/

#include <stdio.h>

#include "bitmap.h"
#include "pstampr.h"

static int rowlen, adiwidth, adiheight;
static unsigned char *opixel;
#ifdef NEEDED
int tmrowlen, tmhm1;
unsigned char *tpixel;
#endif

/*  PSRINIT  --  Specify the address and dimensions of the bitmap
                 into which we shall draw.  */

void psrinit(unsigned char *bits, int width, int height)
{
    adiwidth = width;
    adiheight = height;
    opixel = bits;
    rowlen = ((width + (sizeof(LONG) - 1)) / sizeof(LONG)) * sizeof(LONG);
}

/*  PSRVECTOR  --  Draw a vector into the current image array.  */

#define Opix(x, y) ((opixel + ((x) + (((adiheight - 1) - (y)) * (rowlen)))))
#define PixA(x, y) ((tpixel + ((x) + (((DWORD) (tmhm1 - (y))) * (tmrowlen)))))

void psrvector(int fx, int fy,
               int tx, int ty, int colour)
{
    int cfx, cfy, ctx, cty;
    int x, y, m, f, xinc, loopcnt, dx, dy;

#ifdef FlipY
    fy = (iarray[1] - 1) - fy;
    ty = (iarray[1] - 1) - ty;
#endif

    if (fy < ty) {
        cfx = fx;
        cfy = fy;
        ctx = tx;
        cty = ty;
    } else {
        cfx = tx;
        cfy = ty;
        ctx = fx;
        cty = fy;
    }
    dy = cty - cfy;
    dx = ctx - cfx;

    if (dx < 0) {
        xinc = -1;
        dx = -dx;
    } else  {
        xinc = 1;
    }

    x = cfx;
    y = cfy;

    if (dx > dy) {
        f = dy - dx / 2;
        m = dx - dy;
        loopcnt = dx + 1;

        while (loopcnt--) {
            if (x >= 0 && x < adiwidth && y >= 0 && y < adiheight) {
                *Opix(x, y) = (unsigned char) colour;
            }
            x += xinc;
            if (f > 0) {
                f -= m;
                y++;
            } else {
                f += dy;
            }
        }
    } else {
        f = -(dy / 2 - dx);
        m = dy - dx;
        loopcnt = dy + 1;

        while (loopcnt--) {
            if (x >= 0 && x < adiwidth && y >= 0 && y < adiheight) {
                *Opix(x, y) = (unsigned char) colour;
            }
            y++;
            if (f > 0) {
                x += xinc;
                f -= m;
            } else {
                f += dx;
            }
        }
    }
}

/*  PSRFILLRECT  --  Fill rectangle with a given colour.  */

void psrfillrect(int fx, int fy,
                 int tx, int ty, int colour)
{
    int x, y;

#ifdef FlipY
    fy = (iarray[1] - 1) - fy;
    ty = (iarray[1] - 1) - ty;
#endif

    if (tx < fx) {
        x = tx;
        tx = fx;
        fx = x;
    }

    if (ty < fy) {
        y = ty;
        ty = fy;
        fy = y;
    }

    for (x = fx; x <= tx; x++) {
        for (y = fy; y <= ty; y++) {
            *Opix(x, y) = (unsigned char) colour;
        }
    }
}

#ifdef NEEDED

/*  PSRICON  --  Plot an icon at a specified position in the bitmap.  */

void psricon(int iconNo, int ix, int iy)
{
    int t = iconNo * 32, x, y;

/*
fprintf(stderr, "psricon(%d, %d, %d) \n", iconNo, ix, iy);
fprintf(stderr, "  tmhm1 = %d, tmrowlen = %d\n", tmhm1, tmrowlen);
*/
    for (x = 0; x < 32; x++) {
        int ax = ix + x;

        if (ax >= 0 && ax < adiwidth) {
            for (y = 0; y < 32; y++) {
                int ay = iy + y;

                if (ay >= 0 && ay < adiheight) {
                    unsigned char p = *PixA(x + t, y);

                    if (p != transparent) {
                        *Opix(ax, ay) = p;
                    }
                }
            }
        }
    }
}
#endif
