/*

            Trend Analysis

*/

#include <stdio.h>
#include <assert.h>

#include "database.h"
#include "trendfit.h"


/*  calcTrend  --  Calculate trend slope for a given interval.  */

static void calcTrend(FormPtr f, Word rowLabel, int dayOrMonthCount,
                      UInt weightX, UInt calX, char *wunits, DateTimeType *today)
{
    ControlPtr lp;
    signed long weeklyTrendChange, whole, frac;
    char deltaw[24], deltacal[24];
    char sgn;
    long dcal;
    double tslope;
    Boolean tok, mslope = false;
    SWord itemY;
    DateTimeType strend;
    RectangleType r;

    lp = FrmGetObjectPtr(f, FrmGetObjectIndex(f, rowLabel));
    itemY = lp->bounds.topLeft.y;
    
    /*  dayOrMonthCount is positive to indicate a number of days and negative
        to denote a number of months.  If a number of months is given, we back
        up to the corresponding day the given number of months earlier, then
        adjust the day to the end of the month if there are fewer days in the
        start month than the present day in the end month.  */
    
    strend = *today;
    if (dayOrMonthCount > 0) {
        TimAdjust(&strend, -(dayOrMonthCount * (24L * 60 * 60)));
    } else {
        while (dayOrMonthCount < 0) {
            if (strend.month == 1) {
                strend.month = 12;
                strend.year--;
            } else {
                strend.month--;
            }
            dayOrMonthCount++;
        }
        if (strend.day > DaysInMonth(strend.month, strend.year)) {
            strend.day = DaysInMonth(strend.month, strend.year);
        }
    }
    tok = fitSlopeToMultiMonthTrend(strend.year, strend.month - 1, strend.day - 1,
                                    today->year, today->month - 1, today->day - 1, &tslope);

    if (tok) {
        dcal = (long) (CaloriesPerWeightUnit(prefs.weightUnit) * tslope);
        weeklyTrendChange = (signed long) (tslope * 1000000 * 7);
        weeklyTrendChange = cvtLong(weeklyTrendChange, prefs.weightUnit, prefs.displayUnit);
    
        sgn = (weeklyTrendChange < 0) ? '-' : '+';
        if (weeklyTrendChange < 0) {
            weeklyTrendChange = -weeklyTrendChange;
        }
        whole = weeklyTrendChange  / 1000000L;
        frac = ((weeklyTrendChange + 5000L) / 10000L) % 100;
        if (whole == 0 && frac == 0) {
            sgn = '=';
        }
        StrPrintF(deltaw, "%c%ld.%02ld", sgn, whole, frac);
        StrPrintF(deltacal, "%c%ld", (dcal == 0) ? '=' : (dcal > 0 ? '+' : '-'), abs(dcal));
    
        RctSetRectangle(&r, weightX, itemY, 36, 15);
        drawInFieldRight(deltaw, &r);
        
        RctSetRectangle(&r, calX, itemY, 30, 15);
        drawInFieldRight(deltacal, &r);
    } else {
    
        /*  If there is insufficient data in the database to compute a
            trend of this length, blank its label.  */

        RctSetRectangle(&r, 0, itemY, weightX, 15);
        WinEraseRectangle(&r, 0);
    }

}
